// ==============================================================
// Name:		builtins.cpp
//
// Description:	built-in functions
//
// Copyright (C) 2001-2008, J. Martin Wehlou, Man in The Middle AB
// martin@wehlou.com, martin@mitm.se, MiTM AB, Flintstensvagen 10, 75267 Uppsala, Sweden
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// $LastChangedDate: $
// $LastChangedRevision: $
// $LastChangedBy: $
// $HeadURL: $
// $Id: $
// ==============================================================

#include "stdafx.h"
#include <time.h>
#pragma warning(disable:4786)
#include "valstack.h"
#include "symbols.h"
#include "MSXMLImport.h"
#include "funcs.h"
#include "exceptions.h"
#include "constants.h"
#include "builtins.h"

using m::XML_DocumentPtr;
using m::XML_NodePtr;
using std::wcout;
using std::endl;

// ------------------------------------------------------------------------- //

//#define BUILTINLOAD(langname, vtReturns, iParams, classname) \
//	symbols.defineFunc(langname, vtReturns, iParams, new classname());

t_vpVtStr mTS(eValType vt1, eValType vt2, eValType vt3)
{
	t_vpVtStr vpVtStr;
	if (vt1 != vtNONE) vpVtStr.push_back(make_pair(vt1, wstring(L"")));
	if (vt2 != vtNONE) vpVtStr.push_back(make_pair(vt2, wstring(L"")));
	if (vt3 != vtNONE) vpVtStr.push_back(make_pair(vt3, wstring(L"")));
	return vpVtStr;
}


#define BUILTINLOAD(langname, classname, vtReturns, vpVtStr) \
	symbols.defineFunc(langname, new classname(langname, vtReturns, vpVtStr));

void CBuiltins::loadsymbols(CSymbols& symbols)
{
	// string functions
	BUILTINLOAD(L"ucase",		CF_ucase,		vtSTR,		mTS(vtSTR))
	BUILTINLOAD(L"lcase",		CF_lcase,		vtSTR,		mTS(vtSTR))
	BUILTINLOAD(L"filepath",	CF_filepath,	vtSTR,		mTS(vtSTR))
	BUILTINLOAD(L"filename",	CF_filename,	vtSTR,		mTS(vtSTR))
	BUILTINLOAD(L"fileext",		CF_fileext,		vtSTR,		mTS(vtSTR))
	BUILTINLOAD(L"replace",		CF_replace,		vtSTR,		mTS(vtSTR, vtSTR, vtSTR))
	BUILTINLOAD(L"dropchars",	CF_dropchars,	vtSTR,		mTS(vtSTR, vtSTR))

	// files and dir stuff
	BUILTINLOAD(L"dir",			CF_dir,			vtSET,		mTS(vtSTR))
	BUILTINLOAD(L"filedelete",	CF_filedelete,	vtNONE,		mTS(vtSTR))
	BUILTINLOAD(L"filemove",	CF_filemove,	vtNONE,		mTS(vtSTR, vtSTR))
	BUILTINLOAD(L"filecopy",	CF_filecopy,	vtNONE,		mTS(vtSTR, vtSTR))
	BUILTINLOAD(L"getcurdir",	CF_getcurdir,	vtSTR,		mTS())
	BUILTINLOAD(L"setcurdir",	CF_setcurdir,	vtNONE,		mTS(vtSTR))

	// xml stuff
	
	BUILTINLOAD(L"getnode",		CF_node,		vtNODE,		mTS(vtNODE, vtSTR))
	BUILTINLOAD(L"getnodes",	CF_nodes,		vtNODELIST,	mTS(vtNODE, vtSTR))
	BUILTINLOAD(L"evalattr",	CF_evalattr,	vtSTR,		mTS(vtNODE, vtSTR))
	BUILTINLOAD(L"xmlnew",		CF_xmlnew,		vtNODE,		mTS(vtSTR))
	BUILTINLOAD(L"xmlopen",		CF_xmlopen,		vtNODE,		mTS(vtSTR))
	BUILTINLOAD(L"xmlsaveas",	CF_xmlsaveas,	vtNONE,		mTS(vtNODE, vtSTR))
	BUILTINLOAD(L"setattr",		CF_setattr,		vtNONE,		mTS(vtNODE, vtSTR, vtSTR))
	BUILTINLOAD(L"delattr",		CF_delattr,		vtNONE,		mTS(vtNODE, vtSTR))
	BUILTINLOAD(L"addnode",		CF_addnode,		vtNODE,		mTS(vtNODE, vtSTR))
	BUILTINLOAD(L"delnode",		CF_delnode,		vtNONE,		mTS(vtNODE))
	BUILTINLOAD(L"delnodes",	CF_delnodes,	vtNONE,		mTS(vtNODELIST))
	BUILTINLOAD(L"delnodeexpr",	CF_delnodeexpr,	vtNONE,		mTS(vtSTR))
	BUILTINLOAD(L"gettext",		CF_gettext,		vtSTR,		mTS(vtNODE))
	BUILTINLOAD(L"settext",		CF_settext,		vtNONE,		mTS(vtNODE, vtSTR))
	BUILTINLOAD(L"deltext",		CF_deltext,		vtNONE,		mTS(vtNODE))
	BUILTINLOAD(L"transform",	CF_transform,	vtNONE,		mTS(vtNODE, vtNODE))

	// set and map stuff
	BUILTINLOAD(L"setadd",		CF_setadd,		vtSET,		mTS(vtSET, vtSTR))
	BUILTINLOAD(L"unmap",		CF_unmap,		vtSTR,		mTS(vtSTR, vtSTR))
	BUILTINLOAD(L"intersect",	CF_intersect,	vtSET,		mTS(vtSET, vtSET))

	// system functions
	BUILTINLOAD(L"datetime",	CF_datetime,	vtSTR,		mTS(vtSTR))

	// runtime and debug stuff
	BUILTINLOAD(L"message",		CF_message,		vtNONE,		mTS(vtSTR))
}

// ==========================================================================
//  string functions
//
//
// ==========================================================================

BUILTINIMP(CF_ucase)(CValStack* pvs, CSymbols&, CParse* pRootParser)
{
	wstring s = pvs->popStr();
	for (wstring::size_type i = 0; i < s.size(); i++)
		s[i] = toupper(s[i]);
	pvs->push(s);
}

// ------------------------------------------------------------------------- //

BUILTINIMP(CF_lcase)(CValStack* pvs, CSymbols&, CParse* pRootParser)
{
	wstring s = pvs->popStr();
	for (wstring::size_type i = 0; i < s.size(); i++)
		s[i] = tolower(s[i]);
	pvs->push(s);
}

// ------------------------------------------------------------------------- //

BUILTINIMP(CF_filepath)(CValStack* pvs, CSymbols&, CParse* pRootParser)
{
	wstring sInput = pvs->popStr();
	wstring::size_type pos;

	pos = sInput.rfind('\\');
	if (pos != wstring::npos)
	{
		pvs->push(sInput.substr(0, pos + 1));
		return;
	}

	pos = sInput.rfind(':');
	if (pos != wstring::npos)
	{
		pvs->push(sInput.substr(0, pos + 1));
		return;
	}

	pvs->push(wstring());
	return;
}

// ------------------------------------------------------------------------- //

BUILTINIMP(CF_filename)(CValStack* pvs, CSymbols&, CParse* pRootParser)
{
	wstring sInput = pvs->popStr();
	long l;
	for (l = sInput.size() - 1; l >= 0; --l)
	{
		if (sInput[l] == '\\' || sInput[l] == ':')
			break;
	}
	// if nothing is found, l = -1
	sInput = sInput.substr(l + 1);

	// now cut off any extension
	wstring::size_type dotpos = sInput.rfind('.');
	if (dotpos != wstring::npos)
		sInput = sInput.substr(0, dotpos);
	pvs->push(sInput);
}

// ------------------------------------------------------------------------- //

BUILTINIMP(CF_fileext)(CValStack* pvs, CSymbols&, CParse* pRootParser)
{
	wstring sInput = pvs->popStr();
	wstring sOutput;
	long l;
	for (l = sInput.size() - 1; l >= 0; --l)
	{
		if (sInput[l] == '.')
		{
			sOutput = sInput.substr(l + 1);
			break;
		}
		if (sInput[l] == '\\' || sInput[l] == ':')
			break;
	}
	pvs->push(sOutput);
}

// ------------------------------------------------------------------------- //

// string = replace(string sIn, string sFind, string sNew);
BUILTINIMP(CF_replace)(CValStack* pvs, CSymbols&, CParse*)
{
	wstring sNew = pvs->popStr();
	wstring sFind = pvs->popStr();
	wstring sIn = pvs->popStr();

	while(true)
	{
		wstring::size_type pos = sIn.find(sFind);
		if (pos == wstring::npos) break;
		sIn.replace(sIn.begin() + pos, sIn.begin() + pos + sFind.size(), 
			sNew.begin(), sNew.end());
	}
	pvs->push(sIn);
}

// ------------------------------------------------------------------------- //

// string = dropchars(string sIn, string sChars);
BUILTINIMP(CF_dropchars)(CValStack* pvs, CSymbols&, CParse*)
{
	wstring sChars = pvs->popStr();
	wstring sIn = pvs->popStr();

	for(wstring::size_type i = 0; i < sChars.size(); i++)
	{
		while(true)
		{
			int iPos = sIn.find(sChars[i]);
			if (iPos == wstring::npos) break;
			sIn.erase(sIn.begin() + iPos);
		}
	}
	pvs->push(sIn);
}

// ==========================================================================
//  file and directory stuff
// ==========================================================================

// filelist = dir(mask);
BUILTINIMP(CF_dir)(CValStack* pvs, CSymbols&, CParse* pRootParser)
{
	wstring sFileMask = pvs->popStr();
	wstring sPath = f::getPath(sFileMask);

	CVSet vset;

	WIN32_FIND_DATA FFD;
	HANDLE hFindFile = FindFirstFile(sFileMask.c_str(), &FFD);

	// no files found isn't a terrible error in itself
	if (hFindFile != INVALID_HANDLE_VALUE)
	{
		int	iMaxFiles = 0;			// only used in trial version
		for(;;)
		{
			wstring sFileName = FFD.cFileName;
			wstring sFullName = f::pathConcat(sPath, sFileName);

			vset.insert(sFileName);
			if (0 == FindNextFile(hFindFile, &FFD)) break;
		}
		FindClose(hFindFile);
	}
	pvs->push(vset);
}

// ------------------------------------------------------------------------- //
// filedelete(string sFilename);
BUILTINIMP(CF_filedelete)(CValStack* pvs, CSymbols&, CParse* pRootParser)
{
	wstring sFileName = pvs->popStr();
	if (!DeleteFile(sFileName.c_str()))
		throw x::file_error(f::getlastsyserr());
}

// ------------------------------------------------------------------------- //
// filemove(string sOldName, string sNewName);
BUILTINIMP(CF_filemove)(CValStack* pvs, CSymbols&, CParse*)
{
	wstring sNewName = pvs->popStr();
	wstring sOldName = pvs->popStr();
	if (!MoveFile(sOldName.c_str(), sNewName.c_str()))
		throw x::file_error(f::getlastsyserr());
}

// ------------------------------------------------------------------------- //
// filecopy(string sOriginal, string sCopy);
BUILTINIMP(CF_filecopy)(CValStack* pvs, CSymbols&, CParse*)
{
	wstring sCopy = pvs->popStr();
	wstring sOriginal = pvs->popStr();
	if (!CopyFile(sOriginal.c_str(), sCopy.c_str(), TRUE))
		throw x::file_error(f::getlastsyserr());
}

// ------------------------------------------------------------------------- //
// string = getcurdir();
BUILTINIMP(CF_getcurdir)(CValStack* pvs, CSymbols&, CParse*)
{
	wchar_t	buf[1024];
	if (GetCurrentDirectory(sizeof(buf)/sizeof(buf[0]), buf) == 0)
		throw x::file_error(f::getlastsyserr(L"getcurdir(): %s"));
	wstring sPath = buf;
	pvs->push(sPath);
}

// ------------------------------------------------------------------------- //
// setcurdir(strexpr);
BUILTINIMP(CF_setcurdir)(CValStack* pvs, CSymbols&, CParse*)
{
	wstring sDir = pvs->popStr();
	if (!SetCurrentDirectory(sDir.c_str()))
		throw x::file_error(f::getlastsyserr(L"setcurdir() : %s"));
}


// ==========================================================================
//	xml stuff
// ==========================================================================

// node node(parentnode, selectexpression);
BUILTINIMP(CF_node)(CValStack* pvs, CSymbols&, CParse* pRootParser)
{
	XML_NodePtr spRV;
	wstring sSelExpr = pvs->popStr();
	XML_NodePtr	spNode = pvs->popNode();
	try {
		spRV = spNode->selectSingleNode(_bstr_t(sSelExpr.c_str()));
	}
	catch (...) {
		throw x::syntaxerror(L"Syntax error in xpath expression");
	}
	pvs->push(spRV);
}

// ------------------------------------------------------------------------- //

// nodelist nodes(parentnode, selectexpression);
BUILTINIMP(CF_nodes)(CValStack* pvs, CSymbols&, CParse* pRootParser)
{
	XML_NodelistPtr spRV;
	wstring sSelExpr = pvs->popStr();
	XML_NodePtr spNode = pvs->popNode();
	if (NULL == spNode)
		throw x::syntaxerror(L"invalid parent node, please check definition");
	try {
		spRV = spNode->selectNodes(_bstr_t(sSelExpr.c_str()));
	}
	catch (...) {
		throw x::syntaxerror(L"Syntax error in xpath expression");
	}
	pvs->push(CNodelist(spRV));
}

// ------------------------------------------------------------------------- //

// node = xmlnew(string sRootName);
BUILTINIMP(CF_xmlnew)(CValStack* pvs, CSymbols& symbols, CParse* pRootParser)
{
	XML_DocumentPtr spDoc;
	spDoc.CoCreateInstance(__uuidof(MSXML::DOMDocument));
		
	wstring sRootName = pvs->popStr();
	wstring s = L"<?xml version='1.0' ?><" + sRootName + L"/>";
	spDoc->loadXML(_bstr_t(s.c_str()));
	XML_NodePtr spRoot = spDoc->GetdocumentElement();
	pvs->push(spRoot);
}

// ------------------------------------------------------------------------- //

// node xf = openxml(filename);
BUILTINIMP(CF_xmlopen)(CValStack* pvs, CSymbols&, CParse* pRootParser)
{
	wstring sFileName = pvs->popStr();
	XML_DocumentPtr spDoc;
	spDoc = f::openXmlDoc(sFileName);
	XML_NodePtr spRoot = spDoc->GetdocumentElement();
	pvs->push(spRoot);
}

// ------------------------------------------------------------------------- //

// evalattr(<node>, <string>)
BUILTINIMP(CF_evalattr)(CValStack* pvs, CSymbols&, CParse* pRootParser)
{
	wstring sAttrName = pvs->popStr();
	XML_NodePtr	spNode = pvs->popNode();
	wstring sValue = m::GetAttribute(spNode, sAttrName);
	pvs->push(sValue);
}

// ------------------------------------------------------------------------- //

// xmlsaveas(node root, string sFileName);
BUILTINIMP(CF_xmlsaveas)(CValStack* pvs, CSymbols& symbols, CParse* pRootParser)
{
	wstring sFileName = pvs->popStr();
	XML_NodePtr spRoot = pvs->popNode();
	XML_DocumentPtr spDoc = spRoot->GetownerDocument();
	spDoc->save(_variant_t(sFileName.c_str()));
}

// ------------------------------------------------------------------------- //
// setattr(node parent, string sName, string sValue);
// setattr(node parent, string sName, double dblValue);
BUILTINIMP(CF_setattr)(CValStack* pvs, CSymbols&, CParse*)
{
	wstring sValue = pvs->popStr();
	wstring sName = pvs->popStr();
	XML_NodePtr spParent = pvs->popNode();
	m::SetAttribute(spParent, sName.c_str(), sValue);
}

// ------------------------------------------------------------------------- //
// delattr(node parent, string sName);
BUILTINIMP(CF_delattr)(CValStack* pvs, CSymbols&, CParse*)
{
	wstring sName = pvs->popStr();
	XML_NodePtr spParent = pvs->popNode();
	m::DelAttribute(spParent, sName);
}

// ------------------------------------------------------------------------- //
// node = addnode(node parent, string sTag);
BUILTINIMP(CF_addnode)(CValStack* pvs, CSymbols& symbols, CParse* pRootParser)
{
	wstring sTag = pvs->popStr();
	XML_NodePtr spParent = pvs->popNode();

	XML_DocumentPtr spDoc = spParent->GetownerDocument();
	XML_NodePtr spNewNode = m::CreateDOMNode(spDoc, sTag);
	spParent->appendChild(spNewNode);
	pvs->push(spNewNode);
}

// ------------------------------------------------------------------------- //
// delnode(node nNode);
BUILTINIMP(CF_delnode)(CValStack* pvs, CSymbols& symbols, CParse* pRootParser)
{
	XML_NodePtr spNode = pvs->popNode();
	XML_NodePtr spParent = spNode->GetparentNode();
	spParent->removeChild(spNode);
}

// ------------------------------------------------------------------------- //
// delnodes(nodelist nNodes);
BUILTINIMP(CF_delnodes)(CValStack* pvs, CSymbols& symbols, CParse* pRootParser)
{
	CNodelist nl = pvs->popNodelist();
	for (CNodelist::iterator itr = nl.begin(); itr != nl.end(); ++itr)
	{
		XML_NodePtr spNode = *itr;
		XML_NodePtr spParent = spNode->GetparentNode();
		spParent->removeChild(spNode);
	}
}

// ------------------------------------------------------------------------- //
// delnodeexpr(node nParent, string sExpression);
BUILTINIMP(CF_delnodeexpr)(CValStack* pvs, CSymbols& symbols, CParse* pRootParser)
{
	wstring sExpr = pvs->popStr();
	XML_NodePtr spParent = pvs->popNode();
	XML_NodelistPtr spNodelist = spParent->selectNodes(_bstr_t(sExpr.c_str()));
	
	spNodelist->reset();
	for(;;)
	{
		XML_NodePtr spNode = spNodelist->nextNode();
		if (!spNode.p) break;
		XML_NodePtr spLocalParent = spNode->GetparentNode();
		spParent->removeChild(spNode);
	}
}

// ------------------------------------------------------------------------- //
// string = gettext(node nNode);
BUILTINIMP(CF_gettext)(CValStack* pvs, CSymbols& symbols, CParse* pRootParser)
{
	// if there is no node at all, push an empty string (yes, this can be argued about)
	XML_NodePtr spNode = pvs->popNode();
	if (!spNode) {
		pvs->push(L"");
	}
	else {
		wstring sText = static_cast<wchar_t*>(spNode->Gettext());
		pvs->push(sText);
	}
}


// ------------------------------------------------------------------------- //
// settext(node nParent, string sText);
BUILTINIMP(CF_settext)(CValStack* pvs, CSymbols& symbols, CParse* pRootParser)
{
	wstring sText = pvs->popStr();
	XML_NodePtr spParent = pvs->popNode();
	spParent->Puttext(_bstr_t(sText.c_str()));
}

// ------------------------------------------------------------------------- //
// deltext(node nParent);
BUILTINIMP(CF_deltext)(CValStack* pvs, CSymbols& symbols, CParse* pRootParser)
{
	XML_NodePtr spParent = pvs->popNode();
	spParent->Puttext(L"");
}

// ------------------------------------------------------------------------- //
// transform(node xmlroot, node xslroot);
BUILTINIMP(CF_transform)(CValStack* pvs, CSymbols& symbols, CParse* pRootParser)
{
	XML_NodePtr spXSL = pvs->popNode();
	XML_NodePtr spXML = pvs->popNode();
	spXML->transformNode(spXSL);
}

// ==========================================================================
//	set and map stuff
// ==========================================================================

// set = addset(set, string);
BUILTINIMP(CF_setadd)(CValStack* pvs, CSymbols&, CParse*)
{
	wstring s = pvs->popStr();
	CVSet vSet = pvs->popVSet();
	vSet.insert(s);
	pvs->push(vSet);
}

// ------------------------------------------------------------------------- //

// string = unmap(<mapname>, <key>)
BUILTINIMP(CF_unmap)(CValStack* pvs, CSymbols& symbols, CParse* pRootParser)
{
	wstring sKey = pvs->popStr();
	wstring sMapName = pvs->popStr();
	wstring sResult = symbols.getVMap(sMapName).lookup(sKey);
	pvs->push(sResult);
}

// ------------------------------------------------------------------------- //

// set = intersect(set, set);
BUILTINIMP(CF_intersect)(CValStack* pvs, CSymbols&, CParse*)
{
	CVSet vSet2 = pvs->popVSet();
	CVSet vSet1 = pvs->popVSet();
	CVSet vSet;

	for (CVSet::iterator itr = vSet1.begin(); itr != vSet1.end(); ++itr)
	{
		if (vSet2.isInSet(*itr))
			vSet.insert(*itr);
	}
	pvs->push(vSet);
}

// ==========================================================================
//	system functions
// ==========================================================================

// string = datetime(string sFormat);
BUILTINIMP(CF_datetime)(CValStack* pvs, CSymbols&, CParse*)
{
	wchar_t buf[128];
	wmemset(buf, 0, sizeof(buf)/sizeof(buf[0]));

	wstring sFormat = pvs->popStr();
	if (sFormat.empty())
		sFormat = L"%A %d %b %Y %H:%M:%S";

	time_t timer;
	time(&timer);
	tm tmBuf;
	if (0 != localtime_s(&tmBuf, &timer))
		throw x::syntaxerror(L"Error in datetime format");
	size_t stResult = wcsftime(buf, sizeof(buf)/sizeof(buf[0]), sFormat.c_str(), &tmBuf);

	wstring sOut = buf;

	pvs->push(sOut);
}

// ==========================================================================
//	runtime and debug stuff
// ==========================================================================

// message(stringexpr);
// sends to console output
BUILTINIMP(CF_message)(CValStack* pvs, CSymbols&, CParse*)
{
	wstring sMsg = pvs->popStr();
	_cwprintf(L"%s\n", sMsg.c_str());

//	wcout << sMsg << endl;
}

// ==========================================================================
//  local helpfunctions
// ==========================================================================

