// ==============================================================
// Name:			x2c - builtins.h
//
// Description:		builtin functions
//
// Copyright (C) 2001-2008, J. Martin Wehlou, Man in The Middle AB
// martin@wehlou.com, martin@mitm.se, MiTM AB, Flintstensvagen 10, 75267 Uppsala, Sweden
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// $LastChangedDate: $
// $LastChangedRevision: $
// $LastChangedBy: $
// $HeadURL: $
// $Id: $
// ==============================================================



#ifndef BUILTINS_H
#define BUILTINS_H
#pragma message("--- builtins.h")

#pragma message("- begin includes - " __FILE__)
#include "symbols.h"
#pragma message("- end includes - " __FILE__)
// ----------------------------------------------------------------------- //

static t_vpVtStr mTS(eValType vt1 = vtNONE, eValType vt2 = vtNONE, eValType vt3 = vtNONE);


class CBuiltins
{
public:
	static void loadsymbols(CSymbols& symbols);
};

#define BUILTIN(classname)			\
	class classname : public CFunctorBase {	\
	public: classname(const wstring& sFuncName, eValType vtReturns, t_vpVtStr vpVtStr): \
		CFunctorBase(sFuncName, vtReturns, vpVtStr) {} \
	void operator()(CValStack*, CSymbols&, CParse*);};

#define BUILTINIMP(classname) \
	void classname::operator()



// ==========================================================================
//  string functions
// ==========================================================================

BUILTIN(CF_ucase)
BUILTIN(CF_lcase)
BUILTIN(CF_filepath)
BUILTIN(CF_filename)
BUILTIN(CF_fileext)
BUILTIN(CF_replace)
BUILTIN(CF_dropchars)

// ==========================================================================
//  files and dir stuff
// ==========================================================================

BUILTIN(CF_dir)

BUILTIN(CF_filedelete)
BUILTIN(CF_filemove)
BUILTIN(CF_filecopy)

BUILTIN(CF_getcurdir)
BUILTIN(CF_setcurdir)

// ==========================================================================
//  xml stuff
// ==========================================================================

BUILTIN(CF_node)
BUILTIN(CF_nodes)
BUILTIN(CF_evalattr)
BUILTIN(CF_xmlnew)
BUILTIN(CF_xmlopen)

BUILTIN(CF_xmlsaveas)
BUILTIN(CF_setattr)
BUILTIN(CF_delattr)
BUILTIN(CF_addnode)
BUILTIN(CF_delnode)
BUILTIN(CF_delnodes)
BUILTIN(CF_delnodeexpr)
BUILTIN(CF_gettext)
BUILTIN(CF_settext)
BUILTIN(CF_deltext)
BUILTIN(CF_transform)


// ==========================================================================
//	set and map stuff
// ==========================================================================

BUILTIN(CF_setadd)
BUILTIN(CF_unmap)
BUILTIN(CF_intersect)

// ==========================================================================
//	system functions
// ==========================================================================

BUILTIN(CF_datetime)

// ==========================================================================
//	runtime debug and messages
// ==========================================================================

BUILTIN(CF_message)

// ==========================================================================
//  local helpfunctions
// ==========================================================================


// ----------------------------------------------------------------------- //
#else
#pragma message("--* builtins.h already included")
#endif
// ----------------------------------------------------------------------- //
