// ==============================================================
// Name:			x2c - exceptions.h
//
// Description:		exceptions
//
// Copyright (C) 2001-2008, J. Martin Wehlou, Man in The Middle AB
// martin@wehlou.com, martin@mitm.se, MiTM AB, Flintstensvagen 10, 75267 Uppsala, Sweden
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// $LastChangedDate: $
// $LastChangedRevision: $
// $LastChangedBy: $
// $HeadURL: $
// $Id: $
// ==============================================================

#ifndef EXCEPTIONS_H
#define EXCEPTIONS_H
#pragma message("--- exceptions.h")

#pragma message("- begin includes - " __FILE__)

#include <string>
#include <sstream>
#include "MSXMLImport.h"

#pragma message("- end includes - " __FILE__)

//using std::string;
//using std::stringstream;
using std::wstring;
using std::wstringstream;
using std::endl;

// ----------------------------------------------------------------------- //

namespace x 
{
	wstring v2s(const wchar_t* pszFormat, ...);
	
	// ------------------------------------------------------------------------- //
	
	class exc_base
	{
	public:
		const wchar_t* msg() const { return m_sMsg.c_str(); }
	protected:
		exc_base(const wstring& sMsg) : m_sMsg(sMsg) {};
		exc_base() {};
		void set_msg(const wstring& sMsg) { m_sMsg = sMsg; }
	private:
		wstring	m_sMsg;
	};
	
	// ------------------------------------------------------------------------- //
	
	class cmdline_error : public exc_base
	{
	public:
		cmdline_error() : exc_base(L"") {}
	};

	// ------------------------------------------------------------------------- //
	
	class lexerror : public exc_base
	{
	public:
		lexerror(const wstring& sMsg) : exc_base(sMsg) {}
	};

	// ------------------------------------------------------------------------- //
	
	class syntaxerror : public exc_base
	{
	public:
		syntaxerror(const wstring& sMsg) : exc_base(sMsg) {}
	};

	// ------------------------------------------------------------------------- //
	
	class file_error : public exc_base
	{
	public:
		file_error(const wstring& sMsg) : exc_base(sMsg) {}
	};

	// ------------------------------------------------------------------------- //
	
	class internal_error : public exc_base
	{
	public:
		internal_error(const wstring& sMsg) : exc_base(sMsg) {}
	};

	// ------------------------------------------------------------------------- //
	
	class symbol_error : public exc_base
	{
	public:
		symbol_error(const wstring& sMsg, const wstring& sName) {
			wstring msg = sMsg;
			msg += L", symbol: ";
			msg += sName;
			set_msg(msg);
		}
	};

	// ------------------------------------------------------------------------- //
	
	class xml_error : public exc_base
	{
	public:
		xml_error(m::XML_ParseErrorPtr spPE) : exc_base(L"XML Parsing error")
		{
			_bstr_t bstrError(spPE->reason);

			wstringstream ss;

			ss	<<	L"XML Parsing error:"	<< endl
				<<	L"Code = " << spPE->errorCode << endl
				<<	L"Source = Line: " << spPE->line << L"; Char: " << spPE->linepos << endl
				<<	L"Error description = " << static_cast<wchar_t*>(bstrError) << endl;
			set_msg(ss.str());
		}
	};

	// ------------------------------------------------------------------------- //
	
	class value_error : public exc_base
	{
	public:
		value_error(const wstring& sMsg) : exc_base(sMsg) {}
	};
	
	
	
	// ========================================================================= //
	// non-derived exceptions
	// ========================================================================= //
	
	class wrong_token
	{
	public:
		wrong_token() {}
	};

	// ------------------------------------------------------------------------- //
	
	class wrong_expression
	{
	public:
		wrong_expression() {}
	};

	// ------------------------------------------------------------------------- //
	
	class eof
	{
	public:
		eof() {}
	};

	// ------------------------------------------------------------------------- //

	class stackunderflow
	{
	public:
		stackunderflow() {}
	};
};

// ----------------------------------------------------------------------- //
#else
#pragma message("--* exceptions.h already included")
#endif
// ----------------------------------------------------------------------- //
