// ==============================================================
// Name:		x2c - filelex.h
//
// Description:	input file lexer
//
// Copyright (C) 2001-2008, J. Martin Wehlou, Man in The Middle AB
// martin@wehlou.com, martin@mitm.se, MiTM AB, Flintstensvagen 10, 75267 Uppsala, Sweden
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// $LastChangedDate: $
// $LastChangedRevision: $
// $LastChangedBy: $
// $HeadURL: $
// $Id: $
// ==============================================================

#ifndef FILELEX_H
#define FILELEX_H
#pragma message("--- filelex.h")

#pragma message("- begin includes - " __FILE__)
#include <iostream>
#include <fstream>
#include <vector>
#include <map>
#include "lex.h"
#pragma message("- end includes - " __FILE__)

//using std::ifstream;
//using std::streampos;
using std::vector;
using std::map;
using std::pair;
using std::make_pair;

using std::wifstream;
using std::wstreampos;

// ----------------------------------------------------------------------- //

typedef pair<wstreampos, long>	t_pStpLng;
typedef map<wstring, t_pStpLng>	t_mStrPos;
typedef vector<t_pStpLng>		t_vPos;
typedef vector<long>			t_vLng;
typedef vector<wstring>			t_vStr;

// tracking source file line locations over the "#include" statements
struct srcloc {
	srcloc(long lFirst, const wstring& sFileName) : m_lFirst(lFirst), m_lLast(0), m_sFileName(sFileName) {}
	long	m_lFirst;
	long	m_lLast;
	wstring	m_sFileName;
};
typedef vector<srcloc>		t_vSrcLoc;

class CFileLex : public CLex
{
public:
//	CFileLex(ifstream& is);
	CFileLex(const wstring& sInputFile);
	virtual ~CFileLex();

	void	savepos();
	void	restorepos();
	void	disposepos();
	void	restart();
	void	savepos(const wstring& sPosName);
	void	restorepos(const wstring& sPosName);
	wstring	readToString(const wstring& s);


protected:
	void	skipWhite();
	void	nextLine();
	void	loadCurrentLine();
	void	loadFile(wifstream& is);
	bool	hasInclude(const wchar_t* buf, wstring* psIncludeName);
	long	openInputFile(const wstring& sFileName, wifstream* pis);
	void	closeInputFile(wifstream* pis, long lsl);

	wstring	getPositionDescription() const;

	void	dumpvi();

private:
//	ifstream&	m_is;
	wchar_t		m_buf[1024];

	t_vPos		m_vPos;
	t_mStrPos	m_mNamedPos;

	// new and improved... reads the whole file into a vector of
	// strings and then uses that for input
	t_vStr		m_vi;
	long		m_lvi;		// current row in m_vi (one less than "linenumber")

	// source line tracking
	// each struct contains the index of the m_vi array where the file named in the
	// pair begins and ends
	t_vSrcLoc	m_srcloc;

};

// ----------------------------------------------------------------------- //
#else
#pragma message("--* filelex.h already included")
#endif
// ----------------------------------------------------------------------- //
