// ==============================================================
// Name:		x2c - lex.h
//
// Description:	base for lexers
//
// Copyright (C) 2001-2008, J. Martin Wehlou, Man in The Middle AB
// martin@wehlou.com, martin@mitm.se, MiTM AB, Flintstensvagen 10, 75267 Uppsala, Sweden
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// $LastChangedDate: $
// $LastChangedRevision: $
// $LastChangedBy: $
// $HeadURL: $
// $Id: $
// ==============================================================

#ifndef LEX_H
#define LEX_H
#pragma message("--- lex.h")

#pragma message("- begin includes - " __FILE__)
#include <string>
#include <map>
#include "constants.h"
#pragma message("- end includes - " __FILE__)

//using std::string;
using std::wstring;
using std::map;

// ----------------------------------------------------------------------- //

enum token
{
	tNONE,						/*	nothing at all	*/

	tCODEPAGE,					/*  codepage		*/
	tFUNCTION,					/*	function		*/
	tDECLNODELIST,				/*	nodelist		*/
	tDEFAULT,					/*	default			*/
	tFOREACH,					/*	foreach			*/
//	tINCLUDE,					/*	include			*/
	tPROGRAM,					/*	program			*/
	tUNICODE,					/*  unicode			*/
	tAPPEND,					/*	append			*/
	tCREATE,					/*	create			*/
	tDEFINE,					/*	define			*/
	tDECLDOUBLE,				/*	double			*/
	tIFNDEF,					/*	ifndef			*/
	tOUTPUT,					/*	output			*/
	tRETURN,					/*	return			*/
	tDECLSTRING,				/*	string			*/
	tBREAK,						/*	break			*/
	tCLOSE,						/*	close			*/
	tENDIF,						/*	endif			*/
	tERROR,						/*	error			*/
	tFALSE,						/*	false			*/
	tIFDEF,						/*	ifdef			*/
	tWHERE,						/*	where			*/
	tWHILE,						/*	while			*/
	tDECLBOOL,					/*	bool			*/
	tCOPY,						/*	copy			*/
	tELSE,						/*	else			*/
	tFILE,						/*	file			*/
	tDECLNODE,					/*	node			*/
	tOPEN,						/*	open			*/
	tREAD,						/*	read			*/
	tTRUE,						/*	true			*/
	tFOR,						/*	for				*/
	tDECLMAP,					/*	map				*/
	tDECLSET,					/*	set				*/
	tXML,						/*	xml				*/
	tAS,						/*	as				*/
	tIF,						/*	if				*/
	tIN,						/*	in				*/
	tAND,						/*	&&				*/
	tOR,						/*	||				*/
	tINCREMENT,					/*	++				*/
	tDECREMENT,					/*	--				*/
	tPLUSASS,					/*	+=				*/
	tMINASS,					/*	-=				*/
	tMULTASS,					/*	*=				*/
	tDIVASS,					/*	/=				*/
	tCONCATASS,					/*	&=				*/
	tLE,						/*	<=				*/
	tGE,						/*	>=				*/
	tEQ,						/*	==				*/
	tNE,						/*	!=				*/
	tOPENDSQB,					/*	[[				*/
	tCLOSEDSQB,					/*	]]				*/
	tDSLASH,					/*	//				*/
	tBEGCMT,					/*  / *				*/
	tENDCMT,					/*	* /				*/
	tCOMMA,						/*	,				*/
	tDOT,						/*	.				*/
	tSLASH,						/*	/				*/
	tDIVIDE = tSLASH,			/*	/				*/
	tBACKSLASH,					/*	\				*/ 
	tCOLON,						/*	:				*/
	tSEMICOLON,					/*	;				*/
	tPLUS,						/*	+				*/
	tMINUS,						/*	-				*/
	tMULTIPLY,					/*	*				*/
	tAMPERSAND,					/*	&				*/
	tASSIGN,					/*	=				*/
	tPIPE,						/*	|				*/
	tOPENPAREN,					/*	(				*/
	tCLOSEPAREN,				/*	)				*/
	tOPENBRACE,					/*	{				*/
	tCLOSEBRACE,				/*	}				*/
	tOPENBRACKET,				/*	[				*/
	tCLOSEBRACKET,				/*	]				*/
	tHASH,						/*	#				*/
	tAT,						/*	@				*/
	tEXCLAMATION,				/*	!				*/
	tNOTOP = tEXCLAMATION,		/*	!				*/
	tPERCENT,					/*	%				*/
	tQUESTION,					/*	?				*/
	tLT,						/*	<				*/
	tGT,						/*	>				*/
//	tDQUOTE,					/*	"				*/
//	tSQUOTE,					/*	'				*/

	// derived stuff
	tEOF,						/* end of file		*/
	tVARNAME,					/* variable name	*/
	tNUMBER,					/* number			*/
	tQSTRING,					/* quoted string	*/
};

typedef map<wstring, token>	t_mStrTok;


// CTkn objects are returned to the cpstore class
// for storage in p-code form

class CTkn_sub
{
};

class CTkn
{
public:
	// s is variable name, keyword in full or quoted string
	CTkn(token t) : m_t(t), m_ts(NULL) {}
	CTkn(token t, const wstring& s) : m_t(t), m_s(s), m_ts(NULL) {}
	CTkn(token t, double dbl) : m_t(t), m_dbl(dbl), m_ts(NULL) {}
	virtual ~CTkn();

	token getToken() const				{ return m_t; }
	const wstring& getString() const	{ return m_s; }
	double getDbl() const				{ return m_dbl; }

private:
	token		m_t;
	wstring		m_s;
	double		m_dbl;
	CTkn_sub*	m_ts;
};


// public functions:
//
// peekToken	return current token
// peekVarName	return current variable name or keyword 
// peekNumber	return current number
// peekString	return current quoted string contents
// matchToken	step over current token if same as parameter
// readToken	peek and match
// ifToken		peek and match if same as parameter
// skipToToken	move ahead until param is current token
// skipToEOL	return everything up to EOL and move there
// getErrorInfo
// position functions
// readToString



class CLex
{
public:
	CLex();
	virtual ~CLex();

	token	peekToken();
	token	readToken();
	bool	ifToken(token t);
	wstring	readString();
	double	readNumber();
	wstring	readVarName();
	CTkn	readCTkn();			// special for cpstore
	token	matchToken(token t);
	wstring	readToEOL();
	void	skipToEOL();
	void	skipPastToken(token t);
	void	skipToToken(token t);

	wstring	getErrorInfo() const;

	static	eValType token2vt(token t);

	// stacked positions
	virtual void	savepos() = 0;
	virtual void	restorepos() = 0;
	virtual void	disposepos() = 0;
	virtual void	restart() = 0;

	// named positions (like functions)
	virtual void	savepos(const wstring& sPosName) = 0;
	virtual void	restorepos(const wstring& sPosName) = 0;
	
	virtual wstring	readToString(const wstring& s) = 0;

protected:
	token	scanOperator(const wchar_t** pcp);
	void	scanToken();
	CTkn	scanCTkn();
	void	scanAlpha(const wchar_t** cpEnd, wstring* ps);
	void	scanNumber(const wchar_t** cpEnd, double* pdbl);
	void	scanQString(const wchar_t** cpEnd, wstring* ps);
	token	lookupKeyword(const wstring& s);

	void	setBufPtr(const wchar_t* pBuf);
	void	advanceLastToken();
	void	clearLastToken();
	void	throwLexError(const wstring& sMsg);
	void	loadTokenMap();

	virtual void	skipWhite() = 0;
	virtual wstring	getPositionDescription() const = 0;

private:
	t_mStrTok		m_tokenmap;

	const wchar_t*	m_cpEnd;			// one beyond current token
	token			m_tLast;			// last token
	wstring			m_sLast;			// keyword / varname / qstring contents
	double			m_dblLast;			// number content

protected:
	const wchar_t*	m_pBuf;				// current line
	const wchar_t*	m_cp;				// current position
};

// ----------------------------------------------------------------------- //
#else
#pragma message("--* lex.h already included")
#endif
// ----------------------------------------------------------------------- //
