// ==============================================================
// Name:		x2c - nodelist.cpp
//
// Description:	implements an x2c nodelist data type
//
// Copyright (C) 2001-2008, J. Martin Wehlou, Man in The Middle AB
// martin@wehlou.com, martin@mitm.se, MiTM AB, Flintstensvagen 10, 75267 Uppsala, Sweden
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// $LastChangedDate: $
// $LastChangedRevision: $
// $LastChangedBy: $
// $HeadURL: $
// $Id: $
// ==============================================================

#include "stdafx.h"
#pragma warning(disable:4786)
#include "nodelist.h"


// ==========================================================================
//  iterator
// ==========================================================================

CNodelist::iterator::iterator(CNodelist& rNL, long lIndex) :
m_parent(rNL),
m_lIndex(lIndex)
{}

CNodelist::iterator& CNodelist::iterator::operator++()
{
	++m_lIndex;
	return *this;
}

XML_NodePtr CNodelist::iterator::operator*()
{
	return m_parent.m_vNodes[m_lIndex];
}

bool CNodelist::iterator::operator==(const CNodelist::iterator& rhs)
{
	return m_lIndex == rhs.m_lIndex;
}

bool CNodelist::iterator::operator!=(const CNodelist::iterator& rhs)
{
	return !(*this == rhs);
}

// ==========================================================================
//  CNodelist
// ==========================================================================

CNodelist::CNodelist()
{}

CNodelist::CNodelist(XML_NodePtr spNode, const wstring& sPathExpr)
{
	XML_NodelistPtr spNodelist = spNode->selectNodes(_bstr_t(sPathExpr.c_str()));

	// to enable the iterators to work in parallel, we need to preload all the nodes
	// in a vector
	loadFromNodelist(spNodelist);
}

CNodelist::CNodelist(XML_NodelistPtr spNodelist)
{
	loadFromNodelist(spNodelist);
}

CNodelist::~CNodelist()
{}

CNodelist::iterator CNodelist::begin()
{
	return iterator(*this, 0);
}

CNodelist::iterator CNodelist::end()
{
	return iterator(*this, m_vNodes.size());
}



// ------------------------------------------------------------------------- //

void CNodelist::loadFromNodelist(XML_NodelistPtr spNodelist)
{
	for(;;)
	{
		XML_NodePtr spNode = spNodelist->nextNode();
		if (!spNode) break;
		m_vNodes.push_back(spNode);
	}
}

// ------------------------------------------------------------------------- //

bool CNodelist::empty() const
{
	return m_vNodes.empty();
}