// ==============================================================
// Name:		x2c - parser.h
//
// Description:	parser
//
// Copyright (C) 2001-2008, J. Martin Wehlou, Man in The Middle AB
// martin@wehlou.com, martin@mitm.se, MiTM AB, Flintstensvagen 10, 75267 Uppsala, Sweden
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// $LastChangedDate: $
// $LastChangedRevision: $
// $LastChangedBy: $
// $HeadURL: $
// $Id: $
// ==============================================================

#ifndef PARSE_H
#define PARSE_H
#pragma message("--- parse.h")

#pragma message("- begin includes - " __FILE__)
#include "symbols.h"
#include "valstack.h"
#include <iostream>
#include <fstream>
#pragma message("- end includes - " __FILE__)

using std::wofstream;

// ----------------------------------------------------------------------- //

class CLex;

class CParse
{
public:
	CParse();
	virtual ~CParse();

	void			run				(CLex* pLex);
	void			runsubparser	(CLex* pLex, CParse* pRootParser, CSymbols* pSym, CValStack* pvs, CFunctorBase* pCurrentFunc);

	void			buildFuncTbl	(CLex* pLex);
	void			addFuncTbl		(CLex* pLex);

	void			doStatements	(CLex* pLex, bool bEnable, CValStack* pvs);
	bool			doStatement		(CLex* pLex, bool bEnable, CValStack* pvs);

	void			doForeach		(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doForeachString	(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doForeachNode	(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doIf			(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doFor			(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doWhile			(CLex* pLex, bool bEnable, CValStack* pvs);	
	void			doDefine		(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doIfdef			(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doIfndef		(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doOpen			(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doClose			(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doOutput		(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doVarname		(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doDeclBool		(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doDeclDouble	(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doDeclString	(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doDeclNode		(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doDeclNodelist	(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doDeclMap		(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doDeclSet		(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doDBSQ			(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doDotLine		(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doReturn		(CLex* pLex, bool bEnable, CValStack* pvs);
	void			doBreak			(CLex* pLex, bool bEnable, CValStack* pvs);

protected:
	void			mapExpr			(CLex* pLex, bool bEnable, CValStack* pvs);
	void			setExpr			(CLex* pLex, bool bEnable, CValStack* pvs);
	
	void			assignTo		(CLex* pLex, const wstring& sVarName, bool bEnable, CValStack* pvs, eValType vt);
	void			opAssignTo		(CLex* pLex, const wstring& sVarName, bool bEnable, CValStack* pvs, eValType vt, token t);
	void			pushVar			(CLex* pLex, const wstring& sVarName, CValStack* pvs);

	ofstream&		getOs			() { return m_os; }
	UINT			getCodepage		() { return m_codepage; }
	void			setCodepage		(UINT codepage) { m_codepage = codepage; }

	// V 3
	void			genexpr			(CLex* pLex, CValStack* pvs);
	void			expr			(CLex* pLex, CValStack* pvs);
	void			relterm			(CLex* pLex, CValStack* pvs);
	void			subexpr			(CLex* pLex, CValStack* pvs);
	void			term			(CLex* pLex, CValStack* pvs);
	void			factor			(CLex* pLex, CValStack* pvs);
	wstring			factorVarName	(CLex* pLex, CValStack* pvs);
	void			func			(CLex* pLex, const wstring& sFuncName, CValStack* pvs);

	void			skipgenexpr		(CLex* pLex);
	void			skipexpr		(CLex* pLex);
	void			skiprelterm		(CLex* pLex);
	void			skipsubexpr		(CLex* pLex);
	void			skipterm		(CLex* pLex);
	void			skipfactor		(CLex* pLex);
	void			skipfactorVarName(CLex* pLex);
	void			skipfunc		(CLex* pLex);

	void			evalboolexpr	(CLex* pLex, token t, CValStack* pvs);
	void			evaldblexpr		(CLex* pLex, token t, CValStack* pvs);
	void			evalstrexpr		(CLex* pLex, token t, CValStack* pvs);
	void			evalnodeexpr	(CLex* pLex, token t, CValStack* pvs);
	void			evalnodelistexpr(CLex* pLex, token t, CValStack* pvs);

	wstring			processLine		(CLex* pLex, const wstring& sOutputLine, CValStack* pvs);
	void			outputLine		(CLex* pLex, const wstring& sOutputLine);

	bool			isRelop			(token t) const;
	void			skipParenExpr	(CLex* pLex, CValStack* pvs);

	void			SyntaxError		(CLex* pLex, const wstring& sMsg);

private:
	CSymbols*		m_pSym;
	static CSymbols* m_pRootSym;
	static ofstream m_os;			// only one output for all		
	UINT			m_codepage;		// code page for output file, if zero it implies unicode
	CParse*			m_pRootParser;	// when subparser
	CFunctorBase*	m_pCurrentFunc;	// when subparsing a function
	bool			m_bIsRootParser;
	bool			m_bBreak;		// set when breaking from statement block
	int				m_iBreakable;	// inc when inside foreach or while
};

// ----------------------------------------------------------------------- //
#else
#pragma message("--* parse.h already included")
#endif
// ----------------------------------------------------------------------- //
