// ==============================================================
// Name:		x2c - stringlex.cpp
//
// Description:	lexer for string elements
//
// Copyright (C) 2001-2008, J. Martin Wehlou, Man in The Middle AB
// martin@wehlou.com, martin@mitm.se, MiTM AB, Flintstensvagen 10, 75267 Uppsala, Sweden
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// $LastChangedDate: $
// $LastChangedRevision: $
// $LastChangedBy: $
// $HeadURL: $
// $Id: $
// ==============================================================

#include "stdafx.h"
#include "exceptions.h"
#include "lex.h"
#include "stringlex.h"

// ------------------------------------------------------------------------- //

CStringLex::CStringLex(const wstring& sInput) :
m_sInput(sInput)
{
	restart();
}

CStringLex::~CStringLex()
{}

// ------------------------------------------------------------------------- //

void CStringLex::restart()
{
	setBufPtr(m_sInput.c_str());
}

// ------------------------------------------------------------------------- //

void CStringLex::savepos()
{
	m_vCharPos.push_back(m_cp - m_pBuf);
}

// ------------------------------------------------------------------------- //

void CStringLex::restorepos()
{
	if (m_vCharPos.size() < 1)
		throw x::internal_error(L"stringlex empty position stack");
	m_cp = m_pBuf + m_vCharPos.back();
}

// ------------------------------------------------------------------------- //

void CStringLex::savepos(const wstring& sPosName)
{
	m_mNamedPos[sPosName] = m_cp - m_pBuf;
}

// ------------------------------------------------------------------------- //

void CStringLex::restorepos(const wstring& sPosName)
{
	t_mStrLng::iterator itr = m_mNamedPos.find(sPosName);
	if (itr == m_mNamedPos.end())
		throw x::internal_error(L"restore to non-existent named position in string lex");
	m_cp = m_pBuf + itr->second;
}

// ------------------------------------------------------------------------- //

void CStringLex::disposepos()
{
	if (m_vCharPos.size() < 1)
		throw x::internal_error(L"stringlex position stack underflow");
	m_vCharPos.pop_back();
}

// ------------------------------------------------------------------------- //

void CStringLex::skipWhite()
{
	while (*m_cp != L'\0' && iswspace(*m_cp))
		++m_cp;
	if (*m_cp == L'\0')
		throw x::eof();
}

// ------------------------------------------------------------------------- //

wstring CStringLex::getPositionDescription() const
{
	return L"";
}

// ------------------------------------------------------------------------- //

wstring CStringLex::readToString(const wstring& s)
{
	wstring sRV;

	const wchar_t* cp = wcsstr(m_cp, s.c_str());
	if (cp == NULL)
	{
		sRV += readToEOL();
	}
	else
	{
		sRV += s.substr(0, (cp - m_cp));
		m_cp = cp;
	}
	return sRV;
}
