// ==============================================================
// Name:		x2c - userfunc.cpp
//
// Description:	executes a user defined function
//
// Copyright (C) 2001-2008, J. Martin Wehlou, Man in The Middle AB
// martin@wehlou.com, martin@mitm.se, MiTM AB, Flintstensvagen 10, 75267 Uppsala, Sweden
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// $LastChangedDate: $
// $LastChangedRevision: $
// $LastChangedBy: $
// $HeadURL: $
// $Id: $
// ==============================================================


#include "stdafx.h"
#pragma warning(disable:4786)
#include "symbols.h"
#include "parse.h"
#include "userfunc.h"
#include "exceptions.h"

// ------------------------------------------------------------------------- //

CUserFunc::CUserFunc(const wstring& sFuncName, eValType vtReturns, 
					 t_vpVtStr vpParams, CLex* pLex) :
CFunctorBase(sFuncName, vtReturns, vpParams),
m_pLex(pLex) {}

CUserFunc::~CUserFunc() {}

// ------------------------------------------------------------------------- //

void CUserFunc::operator()(CValStack* pvs, CSymbols& rootSymbols, CParse* pRootParser)
{
	// check if number of parameters match in call and function definition

	if (pvs->getStackSize() != m_vpVtStr.size())
		throw x::syntaxerror(L"number of parameters in function call does not match definition");

	// set to position immediatly following the paramlist
	m_pLex->savepos();				// save callers position
	m_pLex->restorepos(m_sFuncName);

	// create a new symbol table
	CSymbols symbols;
	symbols.setRootSymbols(&rootSymbols);

	// load parameter values using the definitions in reverse
	for (t_vpVtStr::reverse_iterator rItr = m_vpVtStr.rbegin(); rItr != m_vpVtStr.rend(); ++rItr)
	{
		eValType tParamType = rItr->first;
		wstring sParamName = rItr->second;
		switch(tParamType)
		{
		case vtSTR:
			symbols.defineStr(sParamName, pvs->popStr());
			break;

		case vtDBL:
			symbols.defineDbl(sParamName, pvs->popDbl());
			break;

		case vtBOOL:
			symbols.defineBool(sParamName, pvs->popBool());
			break;

		case vtNODE:
			symbols.defineNode(sParamName, pvs->popNode());
			break;

		case vtNODELIST:
			symbols.defineNodelist(sParamName, pvs->popNodelist());
			break;

		case vtSET:
			symbols.defineSet(sParamName) = pvs->popVSet();
			break;

		case vtMAP:
			symbols.defineMap(sParamName) = pvs->popVMap();
			break;

		default:
			break;
		}
	}

	// run a non-root parser for the function
	CParse funcParser;
	funcParser.runsubparser(m_pLex, pRootParser, &symbols, pvs, this);
	
	// restore callers position
	m_pLex->restorepos();	
	m_pLex->disposepos();
}
