// ==============================================================
// Name:		valstack.h
//
// Description:	value stack for expression parsing
//
// Copyright (C) 2001-2008, J. Martin Wehlou, Man in The Middle AB
// martin@wehlou.com, martin@mitm.se, MiTM AB, Flintstensvagen 10, 75267 Uppsala, Sweden
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// $LastChangedDate: $
// $LastChangedRevision: $
// $LastChangedBy: $
// $HeadURL: $
// $Id: $
// ==============================================================


#ifndef VALSTACK_H
#define VALSTACK_H
#pragma message("--- valstack.h")

#pragma message("- begin includes - " __FILE__)
#include <string>
#include <vector>
#include "MSXMLImport.h"
#include "nodelist.h"
#include "vset.h"
#include "vmap.h"
#include "constants.h"
#pragma message("- end includes - " __FILE__)

using std::string;
using std::wstring;
using std::vector;

// ----------------------------------------------------------------------- //
//enum eValType {
//	vtNONE, vtSTR, vtDBL, vtBOOL, vtNODE, vtNODELIST, vtMAP, vtSET
//};

class CValStack
{
public:

	class val
	{
	public:
		val(const val& src);
		val();
		~val();

		val(const wstring& sValue);
		val(double dblValue);
		val(bool bValue);
		val(XML_NodePtr spNode);
		val(const CNodelist& nodelist);
		val(const CVMap& vMap);
		val(const CVSet& vSet);

		val& operator=(const val& rhs);
		
		eValType		getValType() const		{ return m_vt; }
		wstring			getStr() const;	
		double			getDbl() const;	
		bool			getBool() const;
		XML_NodePtr		getNode() const;
		CNodelist		getNodelist() const;
		CVMap			getVMap() const;
		CVSet			getVSet() const;

	private:
		void			copy(const val& src);

	private:
		eValType		m_vt;

		XML_NodePtr		m_spNode;
		wstring			m_s;
		double			m_dbl;
		bool			m_b;
		CNodelistRcp	m_rcNodelist;
		CVMapRcp		m_rcMap;
		CVSetRcp		m_rcSet;
	};



	CValStack();
	~CValStack();

	void			push(const wstring& sValue);
	void			push(double dblValue);
	void			push(bool bValue);
	void			push(XML_NodePtr spNode);
	void			push(const CNodelist& nodelist);
	void			push(const CVMap& vMap);
	void			push(const CVSet& vSet);

	bool			isStr() const;
	bool			isDbl() const;
	bool			isBool() const;
	bool			isNode() const;
	bool			isNodelist() const;
	bool			isVMap() const;
	bool			isVSet() const;
	eValType		getValType() const;
//	bool			isEmpty() const { return m_stack.empty(); }
	int				getStackSize() const;

	void			pop();
	wstring			popStr();
	double			popDbl();
	bool			popBool();
	XML_NodePtr		popNode();
	CNodelist		popNodelist();
	CVMap			popVMap();
	CVSet			popVSet();

protected:
	void		dump();

private:
	vector<val>	m_stack;

};


// ----------------------------------------------------------------------- //
#else
#pragma message("--* valstack.h already included")
#endif
// ----------------------------------------------------------------------- //
