// ==============================================================
// Name:		vfunc.h
//
// Description:	wraps, or converts, a function in x2c to a value object
//				I guess you could say "delegate" nowadays
//
// Copyright (C) 2001-2008, J. Martin Wehlou, Man in The Middle AB
// martin@wehlou.com, martin@mitm.se, MiTM AB, Flintstensvagen 10, 75267 Uppsala, Sweden
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// $LastChangedDate: $
// $LastChangedRevision: $
// $LastChangedBy: $
// $HeadURL: $
// $Id: $
// ==============================================================


#ifndef VFUNC_H
#define VFUNC_H
#pragma message("--- vfunc.h")

#pragma message("- begin includes - " __FILE__)

#include "rcptr.h"
#include <string>
#include <vector>
#include "constants.h"

#pragma message("- end includes - " __FILE__)

//using std::string;
using std::wstring;
using std::vector;
using std::pair;
using std::make_pair;

// ----------------------------------------------------------------------- //

class CSymbols;
class CParse;
class CValStack;


typedef pair<eValType, wstring>	t_pVtStr;
typedef vector<t_pVtStr>		t_vpVtStr;

class CFunctorBase : public rcBase<CFunctorBase>
{
public:
//	CFunctorBase();
	CFunctorBase(const wstring& sFuncName, eValType vtReturns, t_vpVtStr vpVtStr);
	virtual ~CFunctorBase();
	virtual void operator()(CValStack* pvs, CSymbols& symbols, CParse* pRootParser) = 0;
	eValType	getReturnType() const { return m_vtReturns; }
protected:
	wstring		m_sFuncName;
	eValType	m_vtReturns;
	t_vpVtStr	m_vpVtStr;
};
typedef rcPtr<CFunctorBase> CFunctorBaseRcp;


// a CFuncVal represents one function in the global space of the
// program, be it a user defined function or a built-in function
class CFuncVal
{
public:
//	CFuncVal(const string& sFuncName, CFunctorBaseRcp rcpFunctor);
//	CFuncVal(const string& sFuncName, eValType vtReturns,
//			 int iParams, CFunctorBaseRcp rcpFunctor);
	CFuncVal(CFunctorBaseRcp rcpFunctor);
	virtual ~CFuncVal();
	void operator()(CValStack* pvs, CSymbols& symbols, CParse* pRootParser);
private:
//	string			m_sFuncName;
//	eValType		m_vtReturns;	// what the function returns
//	int				m_iParams;		// nb of input parameters
	CFunctorBaseRcp	m_rcpFunctor;
};


// ----------------------------------------------------------------------- //
#else
#pragma message("--* vfunc.h already included")
#endif
// ----------------------------------------------------------------------- //
