// ==============================================================
// Name:		x2c - vmap.cpp
//
// Description:	implements the map data type
//
// Copyright (C) 2001-2008, J. Martin Wehlou, Man in The Middle AB
// martin@wehlou.com, martin@mitm.se, MiTM AB, Flintstensvagen 10, 75267 Uppsala, Sweden
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// $LastChangedDate: $
// $LastChangedRevision: $
// $LastChangedBy: $
// $HeadURL: $
// $Id: $
// ==============================================================


#include "stdafx.h"
#include "exceptions.h"
#include "vmap.h"

using std::wstring;

CVMap::CVMap() : m_deflt(dERR) {}
CVMap::~CVMap() {}

// ------------------------------------------------------------------------- //

CVMap::iterator::iterator(t_mStrStr::iterator itr) :
m_itr(itr) {};

CVMap::iterator::~iterator() {}

bool CVMap::iterator::operator==(const CVMap::iterator& rhs) const
{
	return m_itr == rhs.m_itr;
}

bool CVMap::iterator::operator!=(const CVMap::iterator& rhs) const
{
	return m_itr != rhs.m_itr;
}

CVMap::iterator& CVMap::iterator::operator++()
{
	++m_itr;
	return *this;
}

const wstring& CVMap::iterator::getKey() const
{
	return m_itr->first;
}

const wstring& CVMap::iterator::getValue() const
{
	return m_itr->second;
}



// ------------------------------------------------------------------------- //

void CVMap::add(const wstring& sKey, const wstring& sValue)
{
	if (m_mStrStr.find(sKey) != m_mStrStr.end())
		throw x::syntaxerror(L"redefinition of key in map");
	m_mStrStr.insert(make_pair(sKey, sValue));
}

void CVMap::setDef(const wstring& sDefVal)
{
	m_sDefVal = sDefVal;
	m_deflt = dSTR;
}

void CVMap::setErrDefault()
{
	m_deflt = dERR;
}

void CVMap::setCopyDefault()
{
	m_deflt = dCOPY;
}

wstring CVMap::lookup(const wstring& sKey)
{
	t_mStrStr::iterator itr = m_mStrStr.find(sKey);
	if (itr != m_mStrStr.end())
		return itr->second;
	else if (m_deflt == dCOPY)
		return sKey;
	else if (m_deflt == dERR)
		throw x::value_error(L"key not found in map");
	else if (m_deflt == dSTR)
		return m_sDefVal;
	else
		throw x::internal_error(L"unknown default value in map");
}

CVMap::iterator CVMap::begin()
{
	return iterator(m_mStrStr.begin());
}

CVMap::iterator CVMap::end()
{
	return iterator(m_mStrStr.end());
}