// ==============================================================
// Name:		x2c - vset.h
//
// Description:	implements a set variable type
//
// Copyright (C) 2001-2008, J. Martin Wehlou, Man in The Middle AB
// martin@wehlou.com, martin@mitm.se, MiTM AB, Flintstensvagen 10, 75267 Uppsala, Sweden
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// $LastChangedDate: $
// $LastChangedRevision: $
// $LastChangedBy: $
// $HeadURL: $
// $Id: $
// ==============================================================

#ifndef VSET_H
#define VSET_H

#include <set>
#include <string>
#include "rcptr.h"

using std::set;
using std::string;
using std::wstring;

// --------------------------------------------------
// a set can hold any string value, but only once
// adding the same value a second time is a no-op, not
// an error

typedef set<wstring>		t_sStr;

class CVSet : public rcBase<CVSet>
{
public:
	class iterator
	{
	public:
		iterator(t_sStr::iterator itr);
		virtual ~iterator();
		bool operator!=(const iterator& rhs) const;
		bool operator==(const iterator& rhs) const;
		const wstring& operator*() const;
		iterator& operator++();
	private:
		t_sStr::iterator	m_itr;
	};

	CVSet();
	CVSet(const CVSet& src);
	virtual ~CVSet();

	CVSet&	operator=(const CVSet& src);
	void	insert(const wstring& s);
	void	insert(CVSet& src);
	void	erase(const wstring& s);
	void	erase(CVSet& src);
	bool	isInSet(const wstring& s);

	iterator	begin();
	iterator	end();

private:
	t_sStr		m_set;

};

typedef rcPtr<CVSet> CVSetRcp;

// --------------------------------------------------

#endif