// ==============================================================
// Name:		x2c.cpp
//
// Description:
//
//	Template driven XML processor
//	Takes as input any number of XML files
//	and any number of templates and outputs
//	any number of text files.
//
//	This app is primarily intended to generate code
//	from XML definitions.
//
//	The command line takes as argument the template file
//	to process. That file can include any number of other
//	template files and can open and process any number of
//	xml files
//
// Copyright (C) 2001-2008, J. Martin Wehlou, Man in The Middle AB
// martin@wehlou.com, martin@mitm.se, MiTM AB, Flintstensvagen 10, 75267 Uppsala, Sweden
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// $LastChangedDate: $
// $LastChangedRevision: $
// $LastChangedBy: $
// $HeadURL: $
// $Id: $
// ==============================================================


#include "stdafx.h"
#include "exceptions.h"
#include <iostream>
#include <atlbase.h>
#include "constants.h"
#include "generator.h"

using std::wcout;
using std::endl;

// ------------------------------------------------------------------------- //

wstring a2w(const char* inp) {
	USES_CONVERSION;

	wstring wstr = A2W(inp);
	return wstr;
}


int main(int argc, char* argv[])
{
	try
	{
		wcout << k_pszTitle;
		if (k_bDebug)
			wcout << L"(debug)";
		else
			wcout << L"(release)";
		wcout << endl;
		wcout << L"build " << __DATE__ << " - " << __TIME__ << endl;
		wcout << k_pszCopyright;

		wcout << endl;
		
		if (argc < 2)
			throw x::cmdline_error();
		wstring sInputFile = a2w(argv[1]);

		CoInitialize(NULL);
		{
			CGenerator gen(sInputFile);
			gen.run();
		}
		CoUninitialize();

		return 0;
	}
	catch(const x::cmdline_error&)
	{
		wcout << L" usage: x2c <templatefilename>" << endl;
		return -1;
	}
	catch(const x::file_error& e)
	{
		wcout << L"Abend file error:" << endl;
		wcout << e.msg() << endl;
		return -1;
	}
}


